import time
import struct

from micropython import const

ADDRESS = 0x62

_REINIT = const(0x3646)
_DATAREADY = const(0xE4B8)
_STOP_PERIODIC_MEASUREMENT = const(0x3F86)
_START_LOW_POWER_PERIODIC_MEASUREMENT = const(0x21AC)
_READMEASUREMENT = const(0xEC05)

import machine


def _read_reply(i2c: machine.I2C, address: int, num: int):
    reply = bytearray(num)
    i2c.readfrom_into(address, reply)
    return reply


def _send_command(i2c: machine.I2C, address: int, cmd: int, delay: float = 0) -> None:
    i2c.writeto(address, struct.pack(">H", cmd))
    time.sleep(delay)


def initialise(i2c: machine.I2C, address: int):
    _send_command(i2c, address, _STOP_PERIODIC_MEASUREMENT, delay=0.5)
    _send_command(i2c, address, _REINIT, delay=0.02)
    _send_command(i2c, address, _START_LOW_POWER_PERIODIC_MEASUREMENT)


def read(i2c: machine.I2C, address: int) -> tuple[float, float, float]:
    _send_command(i2c, address, _READMEASUREMENT, delay=0.001)
    co2, temperature, humidity = struct.unpack(">HxHxHx", _read_reply(i2c, address, 9))
    temperature = -45 + 175 * (temperature / 65535)
    humidity /= 65535
    return temperature, humidity, co2


def is_ready(i2c: machine.I2C, address: int) -> bool:
    """Check the sensor to see if new data is available"""
    _send_command(i2c, address, _DATAREADY, delay=0.001)
    state = struct.unpack(">Hx", _read_reply(i2c, address, 3))[0]
    return (state & 3) != 0
